@@ -63,38 +63,7 @@
     #endif
 #endif
 
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
-#define Qt_MiddleButton Qt::MiddleButton
-inline Qt::Orientation wheelEventOrientation(QWheelEvent *we) {
-    if (std::abs(we->angleDelta().x()) < std::abs(we->angleDelta().y()))
-        return Qt::Vertical;
-    else
-        return Qt::Horizontal;
-}
-inline int wheelEventDelta(QWheelEvent *we) {
-    if(wheelEventOrientation(we) == Qt::Vertical)
-        return we->angleDelta().y();
-    else
-        return we->angleDelta().x();
-}
-inline QPoint wheelEventPos(QWheelEvent *we) {
-    return we->position().toPoint();
-}
-#else
-#define Qt_MiddleButton Qt::MidButton
-inline Qt::Orientation wheelEventOrientation(QWheelEvent *we) {
-    return we->orientation();
-}
-inline int wheelEventDelta(QWheelEvent *we) {
-    return we->delta();
-}
-inline QPoint wheelEventPos(QWheelEvent *we) {
-    return we->pos();
-}
-
-#endif
-
+using namespace cv;
 
 //Static and global first
 static GuiReceiver *guiMainThread = NULL;
@@ -229,7 +198,7 @@ void cvSetPropWindow_QT(const char* name,double prop_value)
         Q_ARG(double, prop_value));
 }
 
-void cv::setWindowTitle(const String& winname, const String& title)
+void setWindowTitle_QT(const String& winname, const String& title)
 {
     if (!guiMainThread)
         CV_Error(Error::StsNullPtr, "NULL guiReceiver (please create a window)");
@@ -1074,7 +1043,7 @@ void GuiReceiver::showImage(QString name, void* arr)
 {
     QPointer<CvWindow> w = icvFindWindowByName(name);
 
-    if (!w) //as observed in the previous implementation (W32, GTK or Carbon), create a new window is the pointer returned is null
+    if (!w) //as observed in the previous implementation (W32, GTK), create a new window is the pointer returned is null
     {
         cvNamedWindow(name.toLatin1().data());
         w = icvFindWindowByName(name);
@@ -1611,9 +1580,7 @@ CvWinProperties::CvWinProperties(QString name_paraWindow, QObject* /*parent*/)
     myLayout->setObjectName(QString::fromUtf8("boxLayout"));
     myLayout->setContentsMargins(0, 0, 0, 0);
     myLayout->setSpacing(0);
-#if QT_VERSION < QT_VERSION_CHECK(5, 13, 0)
     myLayout->setMargin(0);
-#endif
     myLayout->setSizeConstraint(QLayout::SetFixedSize);
     setLayout(myLayout);
 
@@ -1991,9 +1958,7 @@ void CvWindow::createBarLayout()
     myBarLayout->setObjectName(QString::fromUtf8("barLayout"));
     myBarLayout->setContentsMargins(0, 0, 0, 0);
     myBarLayout->setSpacing(0);
-#if QT_VERSION < QT_VERSION_CHECK(5, 13, 0)
     myBarLayout->setMargin(0);
-#endif
 }
 
 
@@ -2003,9 +1968,7 @@ void CvWindow::createGlobalLayout()
     myGlobalLayout->setObjectName(QString::fromUtf8("boxLayout"));
     myGlobalLayout->setContentsMargins(0, 0, 0, 0);
     myGlobalLayout->setSpacing(0);
-#if QT_VERSION < QT_VERSION_CHECK(5, 13, 0)
     myGlobalLayout->setMargin(0);
-#endif
     setMinimumSize(1, 1);
 
     if (param_flags == CV_WINDOW_AUTOSIZE)
@@ -2243,7 +2206,7 @@ void CvWindow::icvLoadControlPanel()
             }
             if (t->type == type_CvButtonbar)
             {
-                int subsize = settings.beginReadArray(QString("buttonbar%1").arg(i));
+                int subsize = settings.beginReadArray(QString("buttonbar")+i);
 
                 if ( subsize == ((CvButtonbar*)t)->layout()->count() )
                     icvLoadButtonbar((CvButtonbar*)t,&settings);
@@ -2274,7 +2237,7 @@ void CvWindow::icvSaveControlPanel()
         }
         if (t->type == type_CvButtonbar)
         {
-            settings.beginWriteArray(QString("buttonbar%1").arg(i));
+            settings.beginWriteArray(QString("buttonbar")+i);
             icvSaveButtonbar((CvButtonbar*)t,&settings);
             settings.endArray();
         }
@@ -2434,14 +2397,14 @@ void OCVViewPort::icvmouseHandler(QMouseEvent* evnt, type_mouse_event category,
         flags |= CV_EVENT_FLAG_LBUTTON;
     if(buttons & Qt::RightButton)
         flags |= CV_EVENT_FLAG_RBUTTON;
-    if(buttons & Qt_MiddleButton)
+    if(buttons & Qt::MidButton)
         flags |= CV_EVENT_FLAG_MBUTTON;
 
     if (cv_event == -1) {
         if (category == mouse_wheel) {
             QWheelEvent *we = (QWheelEvent *) evnt;
-            cv_event = ((wheelEventOrientation(we) == Qt::Vertical) ? CV_EVENT_MOUSEWHEEL : CV_EVENT_MOUSEHWHEEL);
-            flags |= (wheelEventDelta(we) & 0xffff)<<16;
+            cv_event = ((we->orientation() == Qt::Vertical) ? CV_EVENT_MOUSEWHEEL : CV_EVENT_MOUSEHWHEEL);
+            flags |= (we->delta() & 0xffff)<<16;
             return;
         }
         switch(evnt->button())
@@ -2454,7 +2417,7 @@ void OCVViewPort::icvmouseHandler(QMouseEvent* evnt, type_mouse_event category,
             cv_event = tableMouseButtons[category][1];
             flags |= CV_EVENT_FLAG_RBUTTON;
             break;
-        case Qt_MiddleButton:
+        case Qt::MidButton:
             cv_event = tableMouseButtons[category][2];
             flags |= CV_EVENT_FLAG_MBUTTON;
             break;
@@ -2602,9 +2565,8 @@ void DefaultViewPort::updateImage(const CvArr* arr)
     }
 
     nbChannelOriginImage = cvGetElemType(mat);
-
-    cvConvertImage(mat, image2Draw_mat, (origin != 0 ? CV_CVTIMG_FLIP : 0) + CV_CVTIMG_SWAP_RB);
-
+    CV_Assert(origin == 0);
+    convertToShow(cv::cvarrToMat(mat), image2Draw_mat);
     viewport()->update();
 }
 
@@ -2810,7 +2772,7 @@ void DefaultViewPort::wheelEvent(QWheelEvent* evnt)
 {
     icvmouseEvent((QMouseEvent *)evnt, mouse_wheel);
 
-    scaleView(wheelEventDelta(evnt) / 240.0, wheelEventPos(evnt));
+    scaleView(evnt->delta() / 240.0, evnt->pos());
     viewport()->update();
 
     QWidget::wheelEvent(evnt);
@@ -2921,19 +2883,18 @@ inline bool DefaultViewPort::isSameSize(IplImage* img1, IplImage* img2)
 void DefaultViewPort::controlImagePosition()
 {
     qreal left, top, right, bottom;
-    qreal factor = 1.0 / param_matrixWorld.m11();
 
     //after check top-left, bottom right corner to avoid getting "out" during zoom/panning
     param_matrixWorld.map(0,0,&left,&top);
 
     if (left > 0)
     {
-        param_matrixWorld.translate(-left * factor, 0);
+        param_matrixWorld.translate(-left,0);
         left = 0;
     }
     if (top > 0)
     {
-        param_matrixWorld.translate(0, -top * factor);
+        param_matrixWorld.translate(0,-top);
         top = 0;
     }
     //-------
@@ -2942,12 +2903,12 @@ void DefaultViewPort::controlImagePosition()
     param_matrixWorld.map(sizeImage.width(),sizeImage.height(),&right,&bottom);
     if (right < sizeImage.width())
     {
-        param_matrixWorld.translate((sizeImage.width() - right) * factor, 0);
+        param_matrixWorld.translate(sizeImage.width()-right,0);
         right = sizeImage.width();
     }
     if (bottom < sizeImage.height())
     {
-        param_matrixWorld.translate(0, (sizeImage.height() - bottom) * factor);
+        param_matrixWorld.translate(0,sizeImage.height()-bottom);
         bottom = sizeImage.height();
     }
 
@@ -3062,7 +3023,7 @@ void DefaultViewPort::drawStatusBar()
 
         if (nbChannelOriginImage==CV_8UC1)
         {
-            //all the channel have the same value (because of cvconvertimage), so only the r channel is dsplayed
+            //all the channel have the same value (because of cv::cvtColor(GRAY=>RGB)), so only the r channel is dsplayed
             centralWidget->myStatusBar_msg->setText(tr("<font color='black'>(x=%1, y=%2) ~ </font>")
                 .arg(mouseCoordinate.x())
                 .arg(mouseCoordinate.y())+
